import Route from '../model/route';
import {getBaseUrlForLocale, getBaseUrlWithoutCountryPath} from "./router";
import {debugLogger as log} from '../logger/log';

/**
 * Module handles deep linking: When the user clicks a cyberport.de link e.g. on mail client, he is asked if he wants to open the link in the Cyberport app.
 * This function receives the input link (e.g. from intent) and the user settings (important for country de/at).
 * Input link can contain cyberport.de with or without subdomains, pdp, root, category ...
 * Input link and user settings are then used to build and return the app domain link to which the user should be redirected inside the app.
 * E.g.
 * From  https://cyberport.de?DEEP=1H62-020                 to  https://app.cyberport.de/content/webshop30_de/de.html?DEEP=1H62-020
 * From  https://www.cyberport.de/apple-und-zubehoer.html   to  https://app.cyberport.de/content/webshop30_de/de/apple-und-zubehoer.html
 */
var currentRoute = new Route(window.location.href);
var deeplinkUrl = null;
var deeplinkFired = false;
var res = null;

class Deeplink {
    /**
     * Function for deep linking: Deep links open external URLs from browser or mail client in the app
     */
    static initUniversalLinks(settings){
        Deeplink.setSettings(settings);
        universalLinks.subscribe('cpShopMain', function (eventData) {
            log.debug("DEEPLINK: FIRED for:"+eventData.url+" on: "+window.location.href);
            deeplinkFired = true;
            var eventDataObject = eventData;
            Deeplink.getEventDataUrl(eventDataObject)
            .then(url => {
                deeplinkUrl = Deeplink.handleDeeplinks(url);
                if (deeplinkUrl != null) {
                    log.debug("DEEPLINKING: !isAppLandingPage fired for: " + window.location.href);
                    log.debug("DEEPLINKING REDIRECTION TO:" + deeplinkUrl);
                    window.location.replace(deeplinkUrl);
                }
            })
            .catch(error => {
                log.debug(error);
            });
        });
    }
    static async getEventDataUrl(eventData){
        var url = await new Promise((resolve,reject)=> {
            var routeInput = new Route(eventData.url);
            if (routeInput.isNewsletterLink()) {
                log.debug("DEEPLINK: NEWSLETTER LINK");
                // Case: Deeplink comes from newsletter and is a 302 redirect. (http://click.mail.cyberport.de/)
                // This block fetches the 302 ResponseHeader.location, which is then used as deeplink.
                cordova.plugin.http.setFollowRedirect(false);
                cordova.plugin.http.clearCookies();
                const options = {method: 'head',timeout:10};
                cordova.plugin.http.sendRequest(routeInput.getFullRoute(), options,
                    function (responseSuccess) {
                        log.debug("DEEPLINK: Newsletter location response success:"+responseSuccess.url);
                        resolve(responseSuccess.url);
                    }, function (responseError) {
                        log.debug("DEEPLINK: Newsletter location response failed for:"+routeInput.getFullRoute());
                        log.debug(JSON.stringify(responseError));
                        if(responseError.status==302){
                            resolve(responseError.headers.location);
                        } else {
                            resolve("https://app.cyberport.de");
                        }
                    });
            } else {
                log.debug("DEEPLINKING.getEventDataUrl: Newsletter NO: " + window.location.href);
                log.debug(JSON.stringify(eventData));
                log.debug("DEEPLINKING.getEventDataUrl Newsletter NO resolve: " + eventData.url);
                resolve(eventData.url);
            }
        });
        return url;
    }

    static setSettings(settings){
        res = settings;
    }

    /**
     * Function receives the input link and then creates and returns the app link for redirection.
     * @param {string} url - Deeplinking plugins provides eventData.url which contains the URL which should be opened
     * @param {object} settings - Contains user settings on device (important for redirection to DE or AT)
     * @returns {string} url - Contains the eventData.url mapped to app domains
     */
    static handleDeeplinks(url){
        log.debug("DEEPLINK.handleDeeplinks URL::"+JSON.stringify(url));
        var routeInput = new Route(url);
        var routeOutput = null;
        if(routeInput.isPathPdpDeeplink()){
            /* Case: PDP Deeplink, e.g.: https://cyberport.de?DEEP=1H62-020 or https://app.cyberport.de?DEEP=1H62-020 */
            var deepSearchString = routeInput.getSearchString().replace("/","");   // returns ?DEEP=1H62-020
            routeOutput = new Route(getBaseUrlForLocale(res.selectedShop) + "de.html" + deepSearchString);
            log.debug("DEEPLINK output (for pdp): "+routeInput.getFullRoute());
        } else if(routeInput.isDirectoryRoot()){
            var deepSearchString = routeInput.getSearchString().replace("/","");
            /* Case: Root, e.g. cyberport.de/ or app.cyberport.de/ */
            routeOutput = new Route(getBaseUrlForLocale(res.selectedShop) + "de.html"+deepSearchString);
            log.debug("DEEPLINK output (for root directory): "+routeInput.getFullRoute());
        } else if(routeInput.isPathAppStartpage()){
            /* Case: Startpage de.html on subdomain, e.g.: https://app.cyberport.de/content/webshop30_de/de.html */
            routeOutput = new Route(getBaseUrlForLocale(res.selectedShop) + "de.html");
            log.debug("DEEPLINK output (for startpage de.html on subdomain): "+routeInput.getFullRoute());
        }  else {
            /* Case: All other paths.
            /* E.g. https://www.cyberport.de/apple-und-zubehoer.html or https://app.cyberport.de/content/webshop30_de/de/apple-und-zubehoer.html
            /* If input path contains webshop30_ path structure, this is removed and afterwards added depending on user settings
             */
            var pathname = routeInput.getPathname();
            var searchString = routeInput.getSearchString();
            pathname = pathname.replace("/content/webshop30_de/de", "");
            pathname = pathname.replace("/content/webshop30_at/de", "");
            routeOutput = new Route(getBaseUrlForLocale(res.selectedShop) + "de" + pathname + searchString);
            log.debug("DEEPLINK output (other parts): pathname:"+pathname);
            log.debug("DEEPLINK output (other parts): searchString:"+searchString);
            log.debug("DEEPLINK output (other parts): "+routeInput.getFullRoute());
        }
        return routeOutput.getFullRoute().replace(".pdp", "/pdp");
    }
}
export {Deeplink, deeplinkUrl, deeplinkFired};
